/*
 * Copyright (c) 2004-2021 Mellanox Technologies LTD. All rights reserved.
 * Copyright (c) 2022-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include "acc_reg_key.h"
#include "phy_diag_types.h"

void AccRegKeyPortLane::DumpKeyData(ostream &stream) const
{
    IBDIAGNET_ENTER;
    char buffer[1024] = {0};
    snprintf(buffer, sizeof(buffer),
             U64H_FMT "," U64H_FMT "," U32D_FMT "," U32D_FMT ",",
             this->node_guid,
             this->port_guid,
             this->port_num,
             this->lane);

    stream << buffer;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyPortLane::DumpKeyHeader(ostream &stream) const
{
    IBDIAGNET_ENTER;
    stream << PORT_LANE_HEADER;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyNode::DumpKeyData(ostream &stream) const
{
    IBDIAGNET_ENTER;
    char buffer[1024] = {0};
    snprintf(buffer, sizeof(buffer),
             U64H_FMT ",",
             this->node_guid);

    stream << buffer;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyNode::DumpKeyHeader(ostream &stream) const
{
    IBDIAGNET_ENTER;
    stream << NODE_HEADER;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyPort::DumpKeyData(ostream &stream) const
{
    IBDIAGNET_ENTER;
    char buffer[1024] = {0};
    snprintf(buffer, sizeof(buffer),
             U64H_FMT "," U64H_FMT "," U32D_FMT ",",
             this->node_guid,
             this->port_guid,
             this->port_num);

    stream << buffer;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyPort::DumpKeyHeader(ostream &stream) const
{
    IBDIAGNET_ENTER;
    stream << PORT_HEADER;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyGroup::DumpKeyData(ostream &stream) const
{
    IBDIAGNET_ENTER;
    char buffer[1024] = {0};
    snprintf(buffer, sizeof(buffer),
             U64H_FMT ",",
             this->node_guid);

    stream << buffer;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyGroup::DumpKeyHeader(ostream &stream) const
{
    IBDIAGNET_ENTER;
    stream << GROUP_HEADER;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyNodeSensor::DumpKeyData(ostream &stream) const
{
    IBDIAGNET_ENTER;
    char buffer[1024] = {0};
    snprintf(buffer, sizeof(buffer),
             U64H_FMT "," U32D_FMT ",",
             this->node_guid,
             this->sensor_id);

    stream << buffer;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyNodeSensor::DumpKeyHeader(ostream &stream) const
{
    IBDIAGNET_ENTER;
    stream << NODE_SENSOR_HEADER;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyTypeGroup::DumpKeyData(ostream &stream) const
{
    IBDIAGNET_ENTER;
    char buffer[1024] = {0};
    snprintf(buffer, sizeof(buffer),
             U64H_FMT ",%u,%u,%u,%u,",
             this->node_guid,
             this->group_type,
             this->group_num,
             this->start_index,
             this->num_of_indices);

    stream << buffer;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyTypeGroup::DumpKeyHeader(ostream &stream) const
{
    IBDIAGNET_ENTER;
    stream << TYPE_GROUP_HEADER;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyDPN::DumpKeyData(ostream &stream) const
{
    IBDIAGNET_ENTER;

    char buffer[256] = {0};
    snprintf(buffer, sizeof(buffer),
             U64H_FMT ",%u,%u,%u,",
             this->node_guid,
             this->pci_idx,
             this->depth,
             this->pci_node);
    stream << buffer;

    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyDPN::DumpKeyHeader(ostream &stream) const
{
    IBDIAGNET_ENTER;
    stream << PCI_DPN_HEADER;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyPCIPortLane::DumpKeyData(ostream &stream) const
{
    IBDIAGNET_ENTER;

    char buffer[256] = {0};
    snprintf(buffer, sizeof(buffer),
             U64H_FMT ",%u,%u,",
             this->node_guid,
             this->port_num,
             this->lane);
    stream << buffer;

    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyPCIPortLane::DumpKeyHeader(ostream &stream) const
{
    IBDIAGNET_ENTER;
    stream << PCI_PORT_LANE_HEADER;
    IBDIAGNET_RETURN_VOID;
}

