 /*
 * Copyright (c) 2022-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef PHY_EXPORT_API_H
#define PHY_EXPORT_API_H

#include "infiniband/ibdiag/ibdiagnet/ibdiagnet_export_api.h"
#include "../internal_packets/internal_packets_layouts.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct
{
    u_int64_t node_guid;
    u_int64_t port_guid;
    u_int8_t port_num;
    // Diagnostic Data Pages
    struct DDPhysCounters *p_dd_phys_counters;
    struct DDTroubleShootingInfo *p_dd_trouble_shooting_info;
    struct DDOperationInfo *p_dd_operation_info;
    struct DDPhyInfo *p_dd_phy_info;
    struct DDModuleInfo *p_dd_module_info;
    struct DDLinkDownInfo *p_dd_link_down_info;
    struct DDLinkUpInfo *p_dd_link_up_info;
    struct DDLatchedFlagInfo *p_dd_latched_flag_info;
    struct DD_RS_Histograms *p_dd_rs_histograms;
    struct DD_PLR_Counters *p_dd_plr_counters;
    struct DD_PhyStatisticalCounters *p_dd_phy_statistical_counters;
    // PHY Registers
    struct slrg_reg *p_slrg_reg[4];
    struct sllm_reg *p_sllm_reg[4];
    struct slrp_reg *p_slrp_reg[4];
    struct sltp_reg *p_sltp_reg[4];
    struct pmdr_reg *p_pmdr_reg;
    struct pphcr_reg *p_pphcr_reg;
    struct ptys_reg *p_ptys_reg;
    struct slrip_reg *p_slrip_reg[4][4];
    struct slsir_reg *p_slsir_reg[4];
    // MEPI layouts
    struct pemi_Module_Status_Samples *p_pemi_module_status_samples;
    struct pemi_SNR_Samples *p_pemi_snr_samples;
    struct pemi_Laser_Monitors_Samples *p_pemi_laser_monitors_samples;
    struct pemi_PAM4_Level_Transition_Samples *p_pemi_pam4_level_transition_samples;
    struct pemi_Pre_FEC_BER_Samples *p_pemi_pre_fec_ber_samples;
    struct pemi_FERC_Samples *p_pemi_ferc_samples;
    struct pemi_Module_Status_Properties *p_pemi_module_status_properties;
    struct pemi_SNR_Properties *p_pemi_snr_properties;
    struct pemi_PAM4_Level_Transition_Properties *p_pemi_pam4_level_transition_properties;
    struct pemi_Laser_Monitors_Properties *p_pemi_laser_monitors_properties;
    struct pemi_Pre_FEC_BER_Properties *p_pemi_pre_fec_ber_properties;
    struct pemi_FERC_Properties *p_pemi_ferc_properties;
} export_data_phy_port_t;

typedef struct
{
    u_int64_t node_guid;
    // Diagnostic Data Pages
    struct DDPageIdentification *p_dd_page_identification;
    struct DDPCIPerfCounters *p_dd_pci_perf_counters;
    struct DD_pcie_timers_states *p_dd_pcie_timers_states;
    struct DD_pcie_lanes_counters *p_dd_pcie_lanes_counters;
    // PHY Registers
    struct ppll_reg *p_ppll_reg[256];
    struct mfsm_reg *p_mfsm_reg[64];
    struct mtmp_reg *p_mtmp_reg[64];
    struct mvcr_reg *p_mvcr_reg[64];
    struct msgi_reg *p_msgi_reg;
    struct pcam_reg *p_pcam_reg;
    // PCI Registers
    struct mpein_reg *p_mpein_reg[4][4][16];
    struct slrg_reg *p_slrg_reg[256];
    struct slrp_reg *p_slrp_reg[256];
    struct sltp_reg *p_sltp_reg[256];
} export_data_phy_node_t;

/*
 * Define EXPORT_IMPL to get function declarations instead of function pointers typedefs.
 * Export library shall define EXPORT_IMPL before including this file:
 *
 * #define EXPORT_IMPL
 * #include "phy_export_api.h"
 */
#ifdef EXPORT_IMPL
#define EXPORT_SYMBOL __attribute__((visibility("default")))

EXPORT_SYMBOL int export_data_phy_port(export_session_handle_t, export_data_phy_port_t *);
EXPORT_SYMBOL int export_data_phy_node(export_session_handle_t, export_data_phy_node_t *);
#else
typedef int (*PF_export_data_phy_port)(export_session_handle_t, export_data_phy_port_t *);
typedef int (*PF_export_data_phy_node)(export_session_handle_t, export_data_phy_node_t *);
#endif

#ifdef __cplusplus
}
#endif

#endif /* PHY_EXPORT_API_H */
