 /*
 * Copyright (c) 2021-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#pragma once

#include <string>

namespace UPHY
{
    typedef enum {
        RegisterUnitFlags_None  = 0x00,
        RegisterUnitFlags_RX    = 0x01,
        RegisterUnitFlags_TX    = 0x02,
        RegisterUnitFlags_CLM   = 0x04
    } register_unit_flags_t;

    typedef enum {
        RegisterAccessFlags_None        = 0x00,
        RegisterAccessFlags_ReadOnly    = 0x01,
        RegisterAccessFlags_WriteOnly   = 0x02,
        RegisterAccessFlags_ReadWrite   = RegisterAccessFlags_ReadOnly | RegisterAccessFlags_WriteOnly,
        RegisterAccessFlags_Proxy       = 0x04,
        RegisterAccessFlags_All         = RegisterAccessFlags_ReadOnly | RegisterAccessFlags_WriteOnly | RegisterAccessFlags_Proxy,
    } register_access_flags_t;

    typedef enum {
        RegisterSecureType_None,
    } register_secure_t;

    typedef enum {
        RegisterFieldType_None  = 0,
        RegisterFieldType_Signed,
        RegisterFieldType_UnSigned,
        RegisterFieldType_Enum,
    } register_field_t;

    typedef enum {
        DataSetType_None,
        DataSetType_DLN,
        DataSetType_CLN
    } dataset_type_t;

    extern const char * to_c_str(register_unit_flags_t value);
    extern const char * to_c_str(register_access_flags_t value);
    extern const char * to_c_str(register_secure_t value);
    extern const char * to_c_str(register_field_t value);
    extern const char * to_c_str(dataset_type_t value);

    template<typename T> std::string to_str(T value) {
        return std::string(to_c_str(value));
    }
}
