 /*
 * Copyright (c) 2021-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#pragma once

#include "uphy_dataset.h"
#include "uphy_data_key.h"

struct peucg_reg;
class CSVOut;

namespace UPHY
{

    class DumpEngine
    {
        public:
            typedef struct peucg_reg                                    block_t;

        private:
            struct less_ptr {
                bool operator()(const AccRegKey * a, const AccRegKey * b) const {
                    return *a < *b;
                }
            };

        private:
            typedef std::map<uint16_t, const block_t*>                  block_map_t;
            typedef std::map<const AccRegKey*, block_map_t, less_ptr>   data_map_t;
            typedef std::map<const DataSet*, data_map_t>                db_map_t;

        private:
            db_map_t m_db;

        public:
            DumpEngine()
            {
            }

            ~DumpEngine()
            {
            }

        public:
            const block_t * add(const DataSet * dataset, const AccRegKey * key, const block_t * data);

        private:
            void build_section_name(const DataSet * dataset, std::string & section) const;

        private:
            void to_csv(std::ostream & stream, const AccRegKey * key, const DataSet::variant_collection_t & variant) const;
            void to_csv(std::ostream & stream, const AccRegKey * key, const block_map_t & blocks, const DataSet::variant_collection_t & variant) const;

        public:
            void to_csv(CSVOut & csv, const std::list<std::string> & dumps) const;
    };
}
